# Copyright (C) 2006 Aaron Bentley
# <aaron.bentley@utoronto.ca>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys

from bzrlib.builtins import cmd_diff
from bzrlib.patches import (hunk_from_header, InsertLine, RemoveLine,
                            ContextLine, Hunk)

from terminal import colorstring

class LineParser(object):
    def parse_line(self, line):
        if line.startswith("@"):
            return hunk_from_header(line)
        elif line.startswith("+"):
            return InsertLine(line[1:])
        elif line.startswith("-"):
            return RemoveLine(line[1:])
        elif line.startswith(" "):
            return ContextLine(line[1:])
        else:
            return line

class DiffWriter(object):
    def __init__(self, target):
        self.target = target
        self.lp = LineParser()
        self.chunks = []

    def write(self, text):
        newstuff = text.split('\n')
        for newchunk in newstuff[:-1]:
            self._writeline(''.join(self.chunks + [newchunk, '\n']))
            self.chunks = []
        self.chunks = [newstuff[-1]]

    def _writeline(self, line):
        item = self.lp.parse_line(line)
        if isinstance(item, Hunk):
            self.target.write(colorstring(str(item), 'blue'))
        elif isinstance(item, InsertLine):
            self.target.write(colorstring(str(item), 'green'))
        elif isinstance(item, RemoveLine):
            self.target.write(colorstring(str(item), 'red', True))
        elif isinstance(item, basestring):
            if item.startswith('==='):
                self.target.write(colorstring(item, 'yellow', False))
            else:
                self.target.write(item)
        else:
            self.target.write(str(item))

    def flush(self):
        self.target.flush()

def colordiff(self, *args, **kwargs):
    real_stdout = sys.stdout
    sys.stdout = DiffWriter(real_stdout)
    try:
        cmd_diff().run(*args, **kwargs)
    finally:
        sys.stdout = real_stdout
